import 'package:meta/meta.dart';
import 'package:path/path.dart';
import 'package:sqflite/sqflite.dart';

export 'package:sqflite/src/database_mixin.dart';

abstract class SQLiteOpenHelper {
  SQLiteOpenHelper(
    this.name,
    this.version,
  );

  @protected
  final String name;
  @protected
  final int version;

  @protected
  Future<Database> _getDatabaseLocked({
    bool readOnly = false,
  }) async {
    String path = join(await getDatabasesPath(), name);
    Database database = await openDatabase(
      path,
      version: version,
      onConfigure: onConfigure,
      onCreate: onCreate,
      onUpgrade: onUpgrade,
      onDowngrade: onDowngrade,
      onOpen: onOpen,
      readOnly: readOnly,
    );
    return database;
  }

  @protected
  Future<Database> getWritableDatabase() {
    return _getDatabaseLocked();
  }

  @protected
  Future<Database> getReadableDatabase() {
    return _getDatabaseLocked(readOnly: true);
  }

  @protected
  Future<void> onConfigure(Database db);

  @protected
  Future<void> onCreate(Database db, int version);

  @protected
  Future<void> onUpgrade(Database db, int oldVersion, int newVersion);

  @protected
  Future<void> onDowngrade(Database db, int oldVersion, int newVersion);

  @protected
  Future<void> onOpen(Database db);

  @protected
  String selectionPlaceholder(List<String> selectionArgs) {
    StringBuffer buffer = StringBuffer();
    for (int i = 0; i < selectionArgs.length; i++) {
      buffer.write(i != 0 ? ',' : '');
      buffer.write('?');
    }
    return buffer.toString();
  }
}

abstract class SimpleSQLiteOpenHelper extends SQLiteOpenHelper {
  SimpleSQLiteOpenHelper(String name, int version) : super(name, version);

  @override
  Future<void> onConfigure(Database db) async {}

  @override
  Future<void> onOpen(Database db) async {}

  @override
  Future<void> onDowngrade(Database db, int oldVersion, int newVersion) async {}
}
